#!/usr/bin/perl -w

# Generate manual pages for sgt-puzzles by running extracts of puzzles.but
# through halibut.

use strict;
use File::Temp;
use IO::File;

# Header information
my $package = "sgt\\-puzzles";
my $date;
my $section = "Games";
my $section_no = "6";

# Name of command corresponding to each game-specific chapter name
my %commands = (blackbox	=> 'blackboxgame',
		bridges         => 'bridges',
		cube		=> 'cube',
		dominosa	=> 'dominosa',
		fifteen		=> 'fifteen',
		flip		=> 'flipgame',
		guess		=> 'guess',
		inertia         => 'inertia',
		lightup         => 'lightup',
		loopy           => 'loopy',
		map             => 'map',
		mines		=> 'mines',
		net		=> 'netgame',
		netslide	=> 'netslide',
		pattern		=> 'pattern',
		pegs		=> 'pegs',
		rectangles	=> 'rect',
		samegame	=> 'samegame',
		sixteen		=> 'sixteen',
		slant           => 'slant',
		solo		=> 'solo',
		tents           => 'tents',
		twiddle		=> 'twiddle',
		untangle	=> 'untangle');
my %short_descs = (blackboxgame	=> 'guessing game',
		   bridges      => 'topological deduction game',
		   cube		=> 'tile manipulation puzzle game',
		   dominosa	=> 'domino puzzle game',
		   fifteen	=> 'tile manipulation puzzle game',
		   flipgame	=> 'tile manipulation puzzle game',
		   guess	=> 'guessing game based on Mastermind',
		   inertia      => 'puzzle game',
		   lightup      => 'puzzle game',
                   loopy        => 'topological deduction game',
		   map          => 'map colouring game',
		   mines	=> 'game based on Minesweeper',
		   netgame	=> 'tile manipulation puzzle game based on FreeNet',
		   netslide	=> 'tile manipulation puzzle game',
		   pattern	=> 'nonogram puzzle game',
		   pegs		=> 'peg solitaire puzzle game',
		   rect		=> 'puzzle game based on Divide by Squares',
		   samegame	=> 'puzzle game',
		   sixteen	=> 'tile manipulation puzzle game',
		   slant        => 'topological deduction game',
		   solo		=> 'puzzle game based on Sudoku',
		   tents        => 'puzzle game',
		   twiddle	=> 'tile manipulation puzzle game',
		   untangle	=> 'puzzle game based on Planarity');

my @MONTHS = qw(January February March April May June
		July August September October November December);

# Chapter name, initialised to dummy value to capture header
my $name = '__HEADER__';

# Contents of each chapter/appendix
my %contents;

# Gather chapters from the original documentation
my $source = new IO::File;
$source->open("<puzzles.but") or die "puzzles.but: $!";
while (<$source>) {
    # Look for chapter/appendix heading
    if (/^\\[AC]{([^}]+)}\s*/) {
	$name = $1;
    }
    # Look for version ID with date
    if (/^\\versionid \$Id: [^ ]+ \d+ (\d{4})-(\d{2})-\d{2} /) {
	$date = "${MONTHS[$2-1]} $1";
    }
    $contents{$name} .= $_;
}
close $source;

# Remove all normal text from the header
$contents{__HEADER__} =~ s/^(?!\\(?:cfg|define|title){).*$//gm;

# Remove introduction from "common features" chapter
$contents{common} =~ s/^.*?(?=\\H{)//s;

for my $chapter (keys %commands) {

    my $command = $commands{$chapter};
    print "Generating $command.6\n";

    my $contents =
      "\\cfg{man-mindepth}{1}\n" # don't show original chapter headings
      . "\\cfg{man-identity}{".uc($command)."}{$section_no}{$date}{$command ($package)}{$section}\n\n"
      . $contents{__HEADER__}
      . "\\C{man-$command} $command\n\n" # dummy chapter
      . "\\H{man-$command-name} NAME\n\n"
      . "\\c{$command} \\- $short_descs{$command}\n\n"
      . "\\H{man-$command-synopsis} SYNOPSIS\n\n"
      . "\\cw{$command} [\\cw{--generate }\\e{n}]\n"
      . "[\\cw{--print }\\e{w}\\cw{x}\\e{h} [\\cw{--with-solutions}]\n"
      . "[\\cw{--scale }\\e{n}] [\\cw{--colour}]]\n"
      . "[\\e{game-parameters}|\\e{game-ID}|\\e{random-seed}]\n\n"
      . "\\cw{$command --version}\n\n"
      . "\\H{man-$command-desc} DESCRIPTION\n\n"
      . $contents{$chapter}
      . $contents{common}
      . "\\H{man-$command-see-also} SEE ALSO\n\n"
      . "Full documentation in /usr/share/doc/$package/puzzles.txt.gz.\n";

    # Kluge cross-references
    sub replace_ref {
	my ($above, $target, $below) = @_;
	# If the target is an earlier or later section in the current page, say
	# it's above or below.
	if ($above =~ m/\\(?:[CHA]|S\d*){$target}/) {
	    'above';
	} elsif ($below =~ m/\\(?:[CHA]|S\d*){$target}/) {
	    'below';
	}
	# Else if the target is a bibliographic entry, include the entry directly.
	elsif ($below =~ m/\\B{$target}\s*(.*?)\s*(?:\\[BCHA]|S\d*|$)/s) {
	    "($1)";
	}
	# Else if it appears to refer to another game, convert to a customary
	# cross-manual-page reference.
	elsif ($target =~ /(\w+)/ && exists $commands{$1}) {
	    "\\e{$commands{$1}}($section_no)";
	}
	# Otherwise (and this shouldn't happen), show the reference target.
	else {
	    print STDERR "Failed to resolve reference to $target\n";
	    $target;
	}
    }
    $contents =~ s/(?:\bin\s+)?\\[kK]{([^}]+)}/replace_ref($`, $1, $')/eg;

    # Run through halibut, working around bug #320333 (fixed for etch,
    # but this source is meant to build under sarge too).
    # halibut does not default to using stdin or stdout, and
    # /dev/std{in,out} apparently don't exist on some systems, so we
    # can't reliably do this with a pipeline.
    my ($temp_but, $temp_but_name) = mkstemp "/tmp/sgt-puzzles-but-XXXXXX"
      or die "$!";
    print $temp_but $contents or die "$!";
    close $temp_but;
    my ($temp_man, $temp_man_name) = mkstemp "/tmp/sgt-puzzles-man-XXXXXX"
      or die "$!";
    close $temp_man;
    system "halibut --man=$temp_man_name $temp_but_name";
    system "grep -v '^\\.UC' <$temp_man_name >$command.6";
    unlink $temp_but_name;
    unlink $temp_man_name;
    -s "$command.6" or die "halibut produced an empty $command.6";
}

exit;
